==> bytecode-infrastructure_2.11-0.8.9-sources.jar: org/opalj/bi/reader/SourceDebugExtension_attributeReader.scala <==
/* BSD 2-Clause License:
 * Copyright (c) 2009 - 2017
 * Software Technology Group
 * Department of Computer Science
 * Technische Universität Darmstadt
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.opalj
package bi
package reader

import java.io.DataInputStream

/**
 * Template method to read in the SourceDebugExtension attribute.
 *
 * The SourceDebugExtension attribute is an optional attribute in the
 * attributes table of a ClassFile structure.
 *
 * @author Michael Eichberg
 */
trait SourceDebugExtension_attributeReader extends AttributeReader {

    type SourceDebugExtension_attribute <: Attribute

    def SourceDebugExtension_attribute(
        constant_pool:        Constant_Pool,
        attribute_name_index: Constant_Pool_Index,
        debug_extension:      Array[Byte]
    ): SourceDebugExtension_attribute

    /**
     * The SourceDebugExtension attribute is an optional attribute in the
     * attributes table of a ClassFile structure.
     *
     * <pre>
     * SourceDebugExtension_attribute {
     *  u2 attribute_name_index;
     *  u4 attribute_length;
     *  u1 debug_extension[attribute_length];
     *  // <- which is a modified UTF 8 String... but – as stated in the spec –
     *  // Note that the debug_extension array may denote a string longer than
     *  // that which can be represented with an instance of class String.
     * }
     * </pre>
     */
    private[this] def parser(
        ap:                   AttributeParent,
        cp:                   Constant_Pool,
        attribute_name_index: Constant_Pool_Index,
        in:                   DataInputStream
    ): SourceDebugExtension_attribute = {
        val attribute_length = in.readInt
        val data = new Array[Byte](attribute_length)
        in.readFully(data)

        SourceDebugExtension_attribute(cp, attribute_name_index, data)
    }

    registerAttributeReader(SourceDebugExtensionAttribute.Name → parser)

}

==> bytecode-infrastructure_2.12-2.0.0-sources.jar: org/opalj/bi/reader/SourceDebugExtension_attributeReader.scala <==
/* BSD 2-Clause License - see OPAL/LICENSE for details. */
package org.opalj
package bi
package reader

import java.io.DataInputStream

/**
 * Template method to read in the SourceDebugExtension attribute.
 *
 * The SourceDebugExtension attribute is an optional attribute in the
 * attributes table of a ClassFile structure.
 */
trait SourceDebugExtension_attributeReader extends AttributeReader {

    //
    // TYPE DEFINITIONS AND FACTORY METHODS
    //

    type SourceDebugExtension_attribute <: Attribute

    def SourceDebugExtension_attribute(
        cp:                   Constant_Pool,
        ap_name_index:        Constant_Pool_Index,
        ap_descriptor_index:  Constant_Pool_Index,
        attribute_name_index: Constant_Pool_Index,
        debug_extension:      Array[Byte]
    ): SourceDebugExtension_attribute

    //
    // IMPLEMENTATION
    //

    /**
     * The SourceDebugExtension attribute is an optional attribute in the
     * attributes table of a ClassFile structure.
     *
     * <pre>
     * SourceDebugExtension_attribute {
     *  u2 attribute_name_index;
     *  u4 attribute_length;
     *  u1 debug_extension[attribute_length];
     *  // <- which is a modified UTF 8 String... but – as stated in the spec –
     *  // Note that the debug_extension array may denote a string longer than
     *  // that which can be represented with an instance of class String.
     * }
     * </pre>
     */
    private[this] def parserFactory() = (
        cp: Constant_Pool,
        ap: AttributeParent,
        ap_name_index: Constant_Pool_Index,
        ap_descriptor_index: Constant_Pool_Index,
        attribute_name_index: Constant_Pool_Index,
        in: DataInputStream
    ) ⇒ {
        val attribute_length = in.readInt
        val data = new Array[Byte](attribute_length)
        in.readFully(data)

        SourceDebugExtension_attribute(
            cp, ap_name_index, ap_descriptor_index, attribute_name_index, data
        )
    }

    registerAttributeReader(SourceDebugExtensionAttribute.Name → parserFactory())

}
